@@ -30,14 +30,17 @@ import os
 import time
 import hashlib
 import functools
+import sqlite3
 import warnings
 from base64 import b64encode
 from dataclasses import dataclass
 from dataclasses import asdict
 
 from gi.repository import GLib
+from gi.repository import Gio
 
 import idna
+from nbxmpp.xmppiri import clean_iri
 from nbxmpp.xmppiri import escape_ifragment
 from nbxmpp.xmppiri import escape_inode
 from nbxmpp.xmppiri import escape_ires
@@ -58,10 +61,10 @@ def ascii_upper(s):
 
 SASL_AUTH_MECHS = [
     'SCRAM-SHA-512-PLUS',
-    'SCRAM-SHA-512',
     'SCRAM-SHA-256-PLUS',
-    'SCRAM-SHA-256',
     'SCRAM-SHA-1-PLUS',
+    'SCRAM-SHA-512',
+    'SCRAM-SHA-256',
     'SCRAM-SHA-1',
     'GSSAPI',
     'PLAIN',
@@ -490,6 +493,31 @@ def deprecation_warning(message):
     warnings.warn(message, DeprecationWarning)
 
 
+def split_jid_string(
+    jid_string: str
+) -> tuple[str | None, str, str | None]:
+
+    # https://tools.ietf.org/html/rfc7622#section-3.2
+
+    # Remove any portion from the first '/' character to the end of the
+    # string (if there is a '/' character present).
+
+    # Remove any portion from the beginning of the string to the first
+    # '@' character (if there is an '@' character present).
+
+    if jid_string.find('/') != -1:
+        rest, resourcepart = jid_string.split('/', 1)
+    else:
+        rest, resourcepart = jid_string, None
+
+    if rest.find('@') != -1:
+        localpart, domainpart = rest.split('@', 1)
+    else:
+        localpart, domainpart = None, rest
+
+    return localpart, domainpart, resourcepart
+
+
 @functools.lru_cache(maxsize=None)
 def validate_localpart(localpart: str) -> str:
     if not localpart or len(localpart.encode()) > 1023:
@@ -634,27 +662,9 @@ class JID:
     @classmethod
     @functools.lru_cache(maxsize=None)
     def from_string(cls, jid_string: str, force_bare: bool = False) -> JID:
-        # https://tools.ietf.org/html/rfc7622#section-3.2
-
-        # Remove any portion from the first '/' character to the end of the
-        # string (if there is a '/' character present).
-
-        # Remove any portion from the beginning of the string to the first
-        # '@' character (if there is an '@' character present).
-
-        if jid_string.find('/') != -1:
-            rest, resourcepart = jid_string.split('/', 1)
-        else:
-            rest, resourcepart = jid_string, None
-
-        if rest.find('@') != -1:
-            localpart, domainpart = rest.split('@', 1)
-        else:
-            localpart, domainpart = None, rest
+        localpart, domainpart, resourcepart = split_jid_string(jid_string)
 
         if force_bare:
-            if localpart is None:
-                raise LocalpartByteLimit
             resourcepart = None
 
         return cls(localpart=localpart,
@@ -694,6 +704,25 @@ class JID:
                    domain=domainpart,
                    resource=None)
 
+    @classmethod
+    @functools.lru_cache(maxsize=None)
+    def from_iri(cls, iri_str: str, *, force_bare: bool = False) -> JID:
+        iri_str = clean_iri(iri_str)
+        localpart, domainpart, resourcepart = split_jid_string(iri_str)
+
+        if localpart is not None:
+            localpart = GLib.Uri.unescape_string(localpart)
+
+        if force_bare:
+            resourcepart = None
+
+        if resourcepart is not None:
+            resourcepart = GLib.Uri.unescape_string(resourcepart)
+
+        return cls(localpart=localpart,
+                   domain=domainpart,
+                   resource=resourcepart)
+
     def __str__(self) -> str:
         if self.localpart:
             jid = f'{self.localpart}@{self.domain}'
@@ -704,21 +733,29 @@ class JID:
             return f'{jid}/{self.resource}'
         return jid
 
+    def __conform__(self, protocol: sqlite3.PrepareProtocol):
+        if protocol is sqlite3.PrepareProtocol:
+            return str(self)
+        raise ValueError
+
     def __hash__(self):
         return hash(str(self))
 
-    def __eq__(self, other: Union[str, JID]) -> bool:
+    def __eq__(self, other: object) -> bool:
         if isinstance(other, str):
             try:
                 return JID.from_string(other) == self
             except Exception:
                 return False
 
-        return (self.localpart == other.localpart and
-                self.domain == other.domain and
-                self.resource == other.resource)
+        if isinstance(other, JID):
+            return (self.localpart == other.localpart and
+                    self.domain == other.domain and
+                    self.resource == other.resource)
 
-    def __ne__(self, other: Union[str, JID]) -> bool:
+        return False
+
+    def __ne__(self, other: object) -> bool:
         return not self.__eq__(other)
 
     def domain_to_ascii(self) -> str:
@@ -1830,14 +1867,15 @@ class Features(Node):
         return self.getTag('mechanisms', namespace=Namespace.SASL) is not None
 
     def has_sasl_2(self):
-        return self.getTag('mechanisms', namespace=Namespace.SASL2) is not None
+        return self.getTag('authentication', namespace=Namespace.SASL2) is not None
 
     def get_mechs(self) -> set[str]:
-        mechanisms = self.getTag('mechanisms', namespace=Namespace.SASL2)
+        mechanisms = self.getTag('authentication', namespace=Namespace.SASL2)
         if mechanisms is None:
             mechanisms = self.getTag('mechanisms', namespace=Namespace.SASL)
-            if mechanisms is None:
-                return set()
+
+        if mechanisms is None:
+            return set()
         mechanisms = mechanisms.getTags('mechanism')
         return set(mech.getData() for mech in mechanisms)
 
@@ -1871,6 +1909,23 @@ class Features(Node):
     def has_anonymous(self):
         return 'ANONYMOUS' in self.get_mechs()
 
+    def get_channel_binding_type(self) -> Optional[Gio.TlsChannelBindingType]:
+        sasl_cb = self.getTag('sasl-channel-binding',
+                              namespace=Namespace.CHANNEL_BINDING)
+        if sasl_cb is None:
+            return None
+
+        exporter = sasl_cb.getTag(
+            'channel-binding', attrs={'type': 'tls-exporter'})
+        if exporter is not None:
+            return Gio.TlsChannelBindingType.EXPORTER
+
+        server_end_point = sasl_cb.getTag(
+            'channel-binding', attrs={'type': 'tls-server-end-point'})
+        if server_end_point is not None:
+            return Gio.TlsChannelBindingType.SERVER_END_POINT
+        return None
+
 
 class ErrorNode(Node):
     """
