@@ -20,6 +20,7 @@ from typing import Any
 from typing import Optional
 
 from gi.repository import GLib
+from gi.repository import Gio
 
 from nbxmpp.namespaces import Namespace
 from nbxmpp.http import HTTPSession
@@ -203,7 +204,7 @@ class Client(Observable):
 
     @property
     def ignored_tls_errors(self):
-        return self._ignored_tls_errors
+        return set(self._ignored_tls_errors)
 
     def set_ignored_tls_errors(self, errors):
         if errors is None:
@@ -222,6 +223,12 @@ class Client(Observable):
     def ciphersuite(self):
         return self._con.ciphersuite
 
+    def get_channel_binding_data(
+        self,
+        type_: Gio.TlsChannelBindingType
+    ) -> Optional[bytes]:
+        return self._con.get_channel_binding_data(type_)
+
     def set_ignore_tls_errors(self, ignore):
         self._ignore_tls_errors = ignore
 
@@ -662,14 +669,14 @@ class Client(Observable):
                 # other connection methods if an error happensafterwards
                 self._connect_successful = True
 
+            self.state = StreamState.WAIT_FOR_FEATURES
+
+        elif self.state == StreamState.WAIT_FOR_FEATURES:
             if self._stream_authenticated and self._mode.is_login_test:
                 self.notify('login-successful')
                 self.disconnect()
                 return
 
-            self.state = StreamState.WAIT_FOR_FEATURES
-
-        elif self.state == StreamState.WAIT_FOR_FEATURES:
             if stanza.getName() != 'features':
                 self._log.error('Invalid response: %s', stanza)
                 self._disconnect_with_error(
