@@ -299,9 +299,13 @@ class ReplyData(NamedTuple):
 
 class ModerationData(NamedTuple):
     stanza_id: str
-    moderator_jid: str
-    reason: Optional[str] = None
-    timestamp: Optional[str] = None
+    moderator_jid: Optional[str]
+    by: Optional[JID]
+    reason: Optional[str]
+    timestamp: Optional[str]
+    stamp: float
+    is_tombstone: bool
+    occupant_id: Optional[str]
 
 
 class DiscoItems(NamedTuple):
@@ -562,7 +566,9 @@ class DiscoInfo(NamedTuple):
     def muc_subjectmod(self) -> Optional[Any]:
         # muc#roominfo_changesubject stems from a wrong example in the MUC XEP
         # Ejabberd and Prosody use this value
+        # muc#roomconfig_changesubject is also used by Prosody
         return (self.get_field_value(Namespace.MUC_INFO, 'muc#roominfo_subjectmod') or
+                self.get_field_value(Namespace.MUC_INFO, 'muc#roomconfig_changesubject') or
                 self.get_field_value(Namespace.MUC_INFO, 'muc#roominfo_changesubject'))
 
     @property
@@ -644,6 +650,13 @@ class DiscoInfo(NamedTuple):
                 return True
         return False
 
+    def has_identity(self, category: str, type_: str) -> bool:
+        for identity in self.identities:
+            if (identity.category == category and
+                    identity.type == type_):
+                return True
+        return False
+
     @property
     def httpupload_max_file_size(self) -> Optional[float]:
         size = self.get_field_value(Namespace.HTTPUPLOAD_0, 'max-file-size')
@@ -769,10 +782,18 @@ class CommonError:
         self.condition_data = self._error_node.getTagData(self.condition)
         self.app_condition = stanza.getAppError()
         self.type = stanza.getErrorType()
+        self.by = None
         self.jid = stanza.getFrom()
         self.id = stanza.getID()
         self._text = {}
 
+        by = self._error_node.getAttr('by')
+        if by is not None:
+            try:
+                self.by = JID.from_string(by)
+            except Exception:
+                pass
+
         text_elements = self._error_node.getTags('text', namespace=Namespace.STANZAS)
         for element in text_elements:
             lang = element.getXmlLang()
@@ -917,7 +938,7 @@ class MAMData(NamedTuple):
     query_id: str
     archive: JID
     namespace: str
-    timestamp: datetime
+    timestamp: float
 
     @property
     def is_ver_1(self) -> bool:
@@ -964,9 +985,11 @@ class MessageProperties:
     type: MessageType = MessageType.NORMAL
     id: Optional[str] = None
     stanza_ids: list[StanzaIDData] = field(default_factory=list)
+    origin_id: Optional[str] = None
     from_: Optional[JID] = None
     to: Optional[JID] = None
     jid: Optional[JID] = None
+    remote_jid: Optional[JID] = None
     subject = None
     body: Optional[str] = None
     thread: Optional[str] = None
@@ -1323,3 +1346,9 @@ class XHTMLData:
         if body is not None:
             return str(body)
         return str(self._bodys.popitem()[1])
+
+
+@dataclass
+class ChannelBindingData:
+    type: str
+    data: bytes
